@@ -27,6 +27,27 @@ use POE::Component::Client::MPD::Status;
 use base qw[ Class::Accessor::Fast ];
 
 # -- MPD interaction: general commands
+
+#
+# event: updatedb( [$path] )
+#
+# Force mpd to rescan its collection. If $path (relative to MPD's music
+# directory) is supplied, MPD will only scan it - otherwise, MPD will rescan
+# its whole collection.
+#
+sub _onpub_updatedb {
+    my $path = defined $_[ARG0] ? $_[ARG0] : '';
+    my $msg = POE::Component::Client::MPD::Message->new( {
+        _from     => $_[SENDER]->ID,
+        _request  => $_[STATE],
+        _answer   => $DISCARD,
+        _commands => [ "update $path" ],
+        _cooking  => $RAW,
+    } );
+    $_[KERNEL]->yield( '_send', $msg );
+}
+
+
 # -- MPD interaction: handling volume & output
 
 #
@@ -284,6 +305,88 @@ sub _onpub_prev {
 }
 
 
+#
+# event: seek( $time, [$song] )
+#
+# Seek to $time seconds in song number $song. If $song number is not specified
+# then the perl module will try and seek to $time in the current song.
+#
+sub _onpub_seek {
+    my ($time, $song) = @_[ARG0, ARG1];
+    $time ||= 0; $time = int $time;
+    my $msg = POE::Component::Client::MPD::Message->new( {
+        _from     => $_[SENDER]->ID,
+        _request  => $_[STATE],
+        _answer   => $DISCARD,
+        _cooking  => $RAW,
+    } );
+
+    if ( defined $song ) {
+        $msg->_commands( [ "seek $song $time" ] );
+    } else {
+        $msg->_pre_from( '_seek_need_current' );
+        $msg->_pre_event( 'status' );
+        $msg->_pre_data( $time );
+    }
+    $_[KERNEL]->yield( '_send', $msg );
+}
+
+
+#
+# event: _seek_need_current( $msg, $current )
+#
+# Use $current to get current song, before sending real seek $msg.
+#
+sub _onpriv_seek_need_current {
+    my ($msg, $current) = @_[ARG0, ARG1];
+    my $song = $current->data->song;
+    my $time = $msg->_pre_data;
+    $msg->_commands( [ "seek $song $time" ] );
+    $_[KERNEL]->yield( '_send', $msg );
+}
+
+
+#
+# event: seekid( $time, [$songid] )
+#
+# Seek to $time seconds in song ID $songid. If $songid number is not specified
+# then the perl module will try and seek to $time in the current song.
+#
+sub _onpub_seekid {
+    my ($time, $song) = @_[ARG0, ARG1];
+    $time ||= 0; $time = int $time;
+    my $msg = POE::Component::Client::MPD::Message->new( {
+        _from     => $_[SENDER]->ID,
+        _request  => $_[STATE],
+        _answer   => $DISCARD,
+        _cooking  => $RAW,
+    } );
+
+    if ( defined $song ) {
+        $msg->_commands( [ "seekid $song $time" ] );
+    } else {
+        $msg->_pre_from( '_seekid_need_current' );
+        $msg->_pre_event( 'status' );
+        $msg->_pre_data( $time );
+    }
+    $_[KERNEL]->yield( '_send', $msg );
+}
+
+
+#
+# event: _seekid_need_current( $msg, $current )
+#
+# Use $current to get current song, before sending real seekid $msg.
+#
+sub _onpriv_seekid_need_current {
+    my ($msg, $current) = @_[ARG0, ARG1];
+    my $song = $current->data->song;
+    my $time = $msg->_pre_data;
+    $msg->_commands( [ "seekid $song $time" ] );
+    $_[KERNEL]->yield( '_send', $msg );
+}
+
+
 1;
 
 __END__
